@@ -6,6 +6,7 @@
 #include <gtk/gtk.h>
 #include <glib/gstdio.h>
 #include <glade/glade.h>   
+#include <libnotify/notify.h>
 
 #include <locale.h>
 
@@ -15,22 +16,24 @@
 #include "assert.h"
 #include "md5.h"
 
+
 /* relative to the home dir */
 #define HOOKS_SEEN ".update-notifier/hooks_seen"
-#define GLOBAL_HOOKS_SEEN "/etc/update-notifier/hooks_seen"
 
-#define HOOK_DEBUG 0
+/* used by e.g. the installer to mark stuff that's already done */
+#define GLOBAL_HOOKS_SEEN "/etc/update-notifier/hooks_seen"
 
 
 // Debuging 
+int HOOK_DEBUG;
+
 static void debug_log_handler (const gchar   *log_domain,
 			       GLogLevelFlags log_level,
 			       const gchar   *message,
 			       gpointer       user_data)
 {
-#if HOOK_DEBUG 
+   if(HOOK_DEBUG > 0)
     g_log_default_handler (log_domain, log_level, message, user_data);
-#endif
 }
 
 
@@ -40,14 +43,12 @@ void hooks_trayicon_update_tooltip (TrayApplet *un, int num_hooks)
    gchar *updates;
    gchar *explanation;
 
-   updates = g_strdup_printf(ngettext("There is %i item of post-update information available!", "There are %i items of post-update information available!", num_hooks),  num_hooks);
-   explanation = ngettext("Press this icon to show the information.", "Press this icon to show the information.", num_hooks);
+   updates = _("Information about newly installed packages available");
+   explanation = _("Click on the notification icon to show the information");
 
    gtk_tooltips_set_tip(GTK_TOOLTIPS(un->tooltip), 
 			GTK_WIDGET (un->eventbox), 
 			updates, explanation);
-
-   g_free (updates);
 }
 
 
@@ -67,7 +68,7 @@ time_t hook_file_time(const gchar *filename)
    struct stat buf;
    char *file = g_strdup_printf("%s/%s",HOOKS_DIR, filename);
    if(g_stat(file, &buf) <0) {
-      g_error("can't stat %s\n",file);
+      g_warning("can't stat %s\n",file);
       g_free(file);
       return 0;
    }
@@ -79,9 +80,9 @@ time_t hook_file_time(const gchar *filename)
    return mtime > ctime ? mtime : ctime;
 }
 
-gboolean hook_file_md5(const gchar *filename, char *md5)
+gboolean hook_file_md5(const gchar *filename, unsigned char *md5)
 {
-   char buf[512];
+   unsigned char buf[512];
    FILE *f;
    char *file;
    int n;
@@ -108,14 +109,14 @@ gboolean hook_file_md5(const gchar *filename, char *md5)
 /* mark a given hook file as seen 
   (actually implemented to write out all the information we have)
 */
-gboolean hook_file_mark_as_seen(HookDialog *hook_dialog, 
+gboolean hook_file_mark_as_seen(HookTrayAppletPrivate *priv, 
 				HookFile *hf)
 {
    g_debug("mark_hook_file_as_seen: %s", hf->filename);
 
    // copy the md5
-   char md5[16];
-   hook_file_md5(hf->filename,&md5);
+   unsigned char md5[16];
+   hook_file_md5(hf->filename, md5);
 
    // mark as seen 
    hf->seen = TRUE;
@@ -137,10 +138,10 @@ gboolean hook_file_mark_as_seen(HookDialog *hook_dialog,
    // as the one just marked (avoids showing duplicated hooks effecivly). 
    //
    // For hooks with the same md5sum use the mtime of the just displayed hook
-   GList *elm = g_list_first(hook_dialog->hook_files);
+   GList *elm = g_list_first(priv->hook_files);
    for(;elm != NULL; elm = g_list_next(elm)) {
       HookFile *e = (HookFile*)elm->data;
-      g_debug("looking at: %s (%s)",e->filename, e->md5);
+      g_debug("will write out: %s (%s)",e->filename, e->md5);
       if(e->seen == TRUE) {
 	 g_debug("e->seen: %s %i %x", e->filename,e->mtime, (int)(e->cmd_run));
 	 fprintf(f,"%s %i %x\n", e->filename, e->mtime, (int)(e->cmd_run));
@@ -158,12 +159,12 @@ gboolean hook_file_mark_as_seen(HookDialog *hook_dialog,
 }
 
 /* mark a given HookFile as run */
-gboolean mark_hook_file_as_run(HookDialog *hook_dialog, HookFile *hf)
+gboolean mark_hook_file_as_run(HookTrayAppletPrivate *priv, HookFile *hf)
 {
    if(hf == NULL)
       return FALSE;
 
-   g_debug("mark_hook_file_as_run: %s\n",hf->filename);
+   g_debug("mark_hook_file_as_run: %s",hf->filename);
    hf->cmd_run = TRUE;
 
    return TRUE;
@@ -258,8 +259,8 @@ gboolean show_next_hook(TrayApplet *ta, GList *hooks)
 {
    //g_debug("show_next_hook()\n");
 
-   HookDialog *hook_dialog = (HookDialog *)ta->user_data;
-   GladeXML *xml = hook_dialog->glade_xml;
+   HookTrayAppletPrivate *priv = (HookTrayAppletPrivate *)ta->user_data;
+   GladeXML *xml = priv->glade_xml;
 
    GtkWidget *text = glade_xml_get_widget(xml, "textview_hook");
    assert(text);
@@ -284,7 +285,7 @@ gboolean show_next_hook(TrayApplet *ta, GList *hooks)
    }
 
    if(hook_file == NULL) {
-      g_debug("no unseen hookfile found in hook list of len (%i) \n",
+      g_debug("no unseen hookfile found in hook list of len (%i)",
 	      g_list_length(hooks));
       return FALSE;
    }
@@ -295,7 +296,7 @@ gboolean show_next_hook(TrayApplet *ta, GList *hooks)
    GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
    FILE *f = fopen(filename, "r");
    if(f == NULL) {
-      g_critical("can't open %s", filename);
+      g_warning("can't open %s", filename);
       g_free(filename);
       return FALSE;
    }
@@ -344,11 +345,11 @@ void cb_button_run(GtkWidget *self, void *data)
    gchar *cmdline;
 
    TrayApplet *ta = (TrayApplet *)data;
-   HookDialog *hook_dialog = (HookDialog *)ta->user_data;
+   HookTrayAppletPrivate *priv = (HookTrayAppletPrivate *)ta->user_data;
 
    /* mark the current hook file as run */
    HookFile *hf = g_object_get_data(G_OBJECT(self), "hook_file");
-   mark_hook_file_as_run(hook_dialog, hf);
+   mark_hook_file_as_run(priv, hf);
 
    gchar *cmd = g_object_get_data(G_OBJECT(self), "cmd");
    gchar *term = g_object_get_data(G_OBJECT(self), "term");
@@ -371,7 +372,7 @@ void cb_button_next(GtkWidget *self, void *data)
 {
    g_debug("cb_button_next()");
    TrayApplet *ta = (TrayApplet *)data;
-   HookDialog *hook_dialog = (HookDialog *)ta->user_data;
+   HookTrayAppletPrivate *priv = (HookTrayAppletPrivate *)ta->user_data;
 
    HookFile *hf;
    hf = (HookFile*)g_object_get_data(G_OBJECT(self),"HookFile");
@@ -381,11 +382,11 @@ void cb_button_next(GtkWidget *self, void *data)
    }
    
    // mark as seen 
-   hook_file_mark_as_seen(hook_dialog, hf);
+   hook_file_mark_as_seen(priv, hf);
    check_update_hooks(ta);
 
-   if(hook_dialog->hook_files != NULL)
-      show_next_hook(ta, hook_dialog->hook_files);
+   if(priv->hook_files != NULL)
+      show_next_hook(ta, priv->hook_files);
 }
 
 
@@ -394,9 +395,9 @@ void cb_button_next(GtkWidget *self, void *data)
 void show_hooks(TrayApplet *ta)
 {
    g_debug("show_hooks()");
-   HookDialog* hook_dialog = (HookDialog *)ta->user_data;
+   HookTrayAppletPrivate* priv = (HookTrayAppletPrivate *)ta->user_data;
 
-   GladeXML *xml = hook_dialog->glade_xml;
+   GladeXML *xml = priv->glade_xml;
    GtkWidget *dia = glade_xml_get_widget(xml, "dialog_hooks");
    assert(dia);
    GtkWidget *button_run = glade_xml_get_widget(xml, "button_run");
@@ -405,7 +406,7 @@ void show_hooks(TrayApplet *ta)
    assert(button_next);
 
    // if show_next_hook() fails for some reason don't do anything
-   if(!show_next_hook(ta, hook_dialog->hook_files))
+   if(!show_next_hook(ta, priv->hook_files))
       return;
 
    int res = gtk_dialog_run(GTK_DIALOG(dia));
@@ -419,7 +420,7 @@ void show_hooks(TrayApplet *ta)
       }
    
       // mark as seen 
-      hook_file_mark_as_seen(hook_dialog, hf);
+      hook_file_mark_as_seen(priv, hf);
       check_update_hooks(ta);
    }
 
@@ -431,7 +432,14 @@ button_release_cb (GtkWidget *widget,
 		   GdkEventButton *event, 
 		   TrayApplet *ta)
 {
+   HookTrayAppletPrivate *priv = (HookTrayAppletPrivate*)ta->user_data;
+
 	if (event->button == 1 && event->type == GDK_BUTTON_RELEASE) {
+	   if(priv->active_notification != NULL) {
+	      notify_notification_close(priv->active_notification, NULL);
+	      priv->active_notification = NULL;
+	   }
+		 
 	   //g_debug("left click on hook applet\n");
 	   show_hooks(ta);
 	}
@@ -440,7 +448,7 @@ button_release_cb (GtkWidget *widget,
 
 gboolean is_hook_relevant(const gchar *hook_file)
 {
-   g_debug("is_hook_relevant()\n");
+   g_debug("is_hook_relevant(): %s", hook_file);
    gboolean res = TRUE;
 
    char *filename = g_strdup_printf("%s%s",HOOKS_DIR,hook_file);
@@ -454,6 +462,7 @@ gboolean is_hook_relevant(const gchar *hook_file)
    // check the DontShowAfterReboot flag
    gchar *b = rfc822_header_lookup(rfc822, "DontShowAfterReboot");
    if(b != NULL && g_ascii_strncasecmp(b, "true",-1) == 0) {
+      g_debug("found DontShowAfterReboot");
 
       // read the uptime information
       double uptime=0, idle=0;
@@ -470,12 +479,13 @@ gboolean is_hook_relevant(const gchar *hook_file)
       time_t mtime = hook_file_time(hook_file);
       time_t now = time(NULL);
 
-      g_debug("now: %i mtime: %i uptime: %f\n",now,mtime,uptime);
-      g_debug("diff: %i  uptime: %f\n",now-mtime,uptime);
+      g_debug("now: %i mtime: %i uptime: %f",now,mtime,uptime);
+      g_debug("diff: %i  uptime: %f",now-mtime,uptime);
       if((int)uptime > 0 && (now - mtime) > (int)uptime) {
-	 g_debug("not relevant because of reboot: %s\n",hook_file);
+	 g_debug("not relevant because of reboot: %s",hook_file);
 	 res = FALSE;
-      }
+      }  else
+	 g_debug("hook is relevant");
    }
    fclose(f);
 
@@ -485,28 +495,59 @@ gboolean is_hook_relevant(const gchar *hook_file)
    return res;
 }
 
+static gboolean show_notification(void *data)
+{
+   TrayApplet *ta = (TrayApplet*)data;
+   HookTrayAppletPrivate *priv = (HookTrayAppletPrivate*)ta->user_data;
+
+   GtkWidget *d = glade_xml_get_widget(priv->glade_xml,"dialog_hooks");
+   if((d && GTK_WIDGET_VISIBLE(d)) || priv->active_notification != NULL)
+      return;
+
+   NotifyNotification *n;
+   GdkPixbuf* pix;
+   n = notify_notification_new(_("Information about newly installed packages available\n"),
+			       _("Click on the notification icon"
+				 " to show the available information.\n"),
+			       NULL, 
+			       GTK_WIDGET(ta->tray_icon));
+   
+   pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), 
+				  GTK_STOCK_DIALOG_INFO, 48,0,NULL);
+   notify_notification_set_icon_from_pixbuf (n, pix);
+   g_object_unref(pix);
+   notify_notification_set_timeout (n, 60*1000);
+   notify_notification_show(n, NULL);
+   priv->active_notification = n;
+   
+   return FALSE;
+}
+
 gboolean check_update_hooks(TrayApplet *ta)
 {
    g_debug("check_update_hooks()");
 
-   HookDialog *hook_dialog = (HookDialog*)ta->user_data;
+   HookTrayAppletPrivate *priv = (HookTrayAppletPrivate*)ta->user_data;
 
    GDir* dir;
    const gchar *hook_file;
 
    dir=g_dir_open(HOOKS_DIR, 0, NULL);
    if(dir == NULL)
-      g_critical("can't read %s directory\n",HOOKS_DIR);
+      g_warning("can't read %s directory\n",HOOKS_DIR);
 
+   g_debug("reading '%s' dir", HOOKS_DIR);
    int unseen_count = 0;
    while((hook_file=g_dir_read_name(dir)) != NULL) {
+      g_debug("investigating file '%s' ", hook_file); 
 
       // check if the hook still applies (for e.g. DontShowAfterReboot)
-      if(!is_hook_relevant(hook_file))
+      if(!is_hook_relevant(hook_file)) {
+	 g_debug("not relevant: '%s'",hook_file);
 	 continue;
-
+      }
       // see if we already know about this hook filename
-      GList *elm = g_list_find_custom(hook_dialog->hook_files,hook_file,
+      GList *elm = g_list_find_custom(priv->hook_files,hook_file,
 				      compare_hook_func);
 
       // not seen before, add to the list
@@ -518,9 +559,9 @@ gboolean check_update_hooks(TrayApplet *ta)
 	 hook_file_md5(hook_file, t->md5);
 	 t->cmd_run = FALSE;
 	 t->seen = FALSE;
-	 hook_dialog->hook_files = g_list_append(hook_dialog->hook_files, t);
+	 priv->hook_files = g_list_append(priv->hook_files, t);
 	 // init elm with the just added record (will be needed below)
-	 elm = g_list_find_custom(hook_dialog->hook_files,hook_file,
+	 elm = g_list_find_custom(priv->hook_files,hook_file,
 				  compare_hook_func);
 	 assert(elm != NULL);
       }
@@ -529,7 +570,6 @@ gboolean check_update_hooks(TrayApplet *ta)
       // availabe already or because we added it)
       HookFile *hf = (HookFile*)elm->data;
 
-
       // file has changed since we last saw it
       time_t new_mtime = hook_file_time(hook_file);
       if(new_mtime > hf->mtime) {
@@ -539,6 +579,7 @@ gboolean check_update_hooks(TrayApplet *ta)
 
       // we have not seen it yet (because e.g. it was just added)
       if(hf->seen == FALSE) {
+	 g_debug("the file '%s' was NOT SEEN yet",hook_file);
 
 	 // update mtime (because we haven't seen the old one and there
 	 // is a new one now)
@@ -551,7 +592,7 @@ gboolean check_update_hooks(TrayApplet *ta)
 	 // if so, we don't increase the unseen count as all identical
 	 // ones will maked as unseen at onces
 	 gboolean md5match = FALSE;
-	 GList *x = g_list_first(hook_dialog->hook_files);
+	 GList *x = g_list_first(priv->hook_files);
 	 while(x!=NULL) {
 	    HookFile *e = (HookFile*)x->data;
 	    if((elm != x)  &&
@@ -568,7 +609,9 @@ gboolean check_update_hooks(TrayApplet *ta)
 	    g_debug("%s increases unseen_count\n",hf->filename);
 	    unseen_count++;
 	 }
-      }
+      } else 
+	 g_debug("already seen: '%s'",hook_file);
+      
 
 
    }
@@ -576,13 +619,19 @@ gboolean check_update_hooks(TrayApplet *ta)
 
    //g_debug("hooks: %i (new: %i)", g_list_length(hook_files), unseen_count);
 
-   GladeXML *xml = hook_dialog->glade_xml;
+   GladeXML *xml = priv->glade_xml;
    GtkWidget *button_next = glade_xml_get_widget(xml, "button_next");
    assert(button_next);
 
+   // show a notification
+   if(unseen_count > 0 && !GTK_WIDGET_VISIBLE(ta->tray_icon)) {
+      g_timeout_add(5000, show_notification, ta);
+   }
+
    switch(unseen_count) {
    case 0:
       gtk_widget_hide(GTK_WIDGET(ta->tray_icon));
+      gtk_widget_unrealize (GTK_WIDGET(ta->tray_icon));
       gtk_widget_hide(button_next);
       break;
    case 1:
@@ -599,7 +648,7 @@ gboolean check_update_hooks(TrayApplet *ta)
    return TRUE;
 }
 
-void hook_read_seen_file(HookDialog *hook_dialog, const char* filename)
+void hook_read_seen_file(HookTrayAppletPrivate *priv, const char* filename)
 {
    char buf[512];
    int time, was_run;
@@ -607,7 +656,7 @@ void hook_read_seen_file(HookDialog *hook_dialog, const char* filename)
    if(f==NULL)
       return;
 
-   g_debug("reading hook_file: %s \n", filename);
+   g_debug("reading hook_file: %s ", filename);
 
    while(fscanf(f, "%s %i %i",buf,&time,&was_run) == 3) {
 
@@ -620,7 +669,7 @@ void hook_read_seen_file(HookDialog *hook_dialog, const char* filename)
 	 continue;
 
       // now check if we already have that filename in the list
-      GList *elm = g_list_find_custom(hook_dialog->hook_files,buf,
+      GList *elm = g_list_find_custom(priv->hook_files,buf,
 				      compare_hook_func);
       if(elm != NULL) {
 	 g_debug("hookfile: %s already in the list", buf);
@@ -653,7 +702,7 @@ void hook_read_seen_file(HookDialog *hook_dialog, const char* filename)
 	 t->seen = TRUE;
 	 hook_file_md5(t->filename,t->md5);
 
-	 hook_dialog->hook_files = g_list_append(hook_dialog->hook_files, t);
+	 priv->hook_files = g_list_append(priv->hook_files, t);
       }
    }
    fclose(f);
@@ -663,18 +712,18 @@ gboolean init_already_seen_hooks(TrayApplet *ta)
 {
    g_debug("init_already_seen_hooks");
 
-   HookDialog* hook_dialog = (HookDialog*)ta->user_data;
+   HookTrayAppletPrivate* priv = (HookTrayAppletPrivate*)ta->user_data;
    char *filename;
 
    // init with default value
-   hook_dialog->hook_files = NULL;  
+   priv->hook_files = NULL;  
 
    // read global hook file
-   hook_read_seen_file(hook_dialog,GLOBAL_HOOKS_SEEN);
+   hook_read_seen_file(priv,GLOBAL_HOOKS_SEEN);
    
    // read user hook file
    filename = g_strdup_printf("%s/%s", g_get_home_dir(),HOOKS_SEEN);
-   hook_read_seen_file(hook_dialog,filename);
+   hook_read_seen_file(priv,filename);
    g_free(filename);
 
 
@@ -685,13 +734,13 @@ gboolean init_already_seen_hooks(TrayApplet *ta)
 
 void hook_tray_icon_init(TrayApplet *ta)
 {
-   HookDialog *hook_dialog = g_new0(HookDialog, 1);
-   ta->user_data = hook_dialog;
+   HookTrayAppletPrivate *priv = g_new0(HookTrayAppletPrivate, 1);
+   ta->user_data = priv;
    
    GladeXML *xml = glade_xml_new(GLADEDIR"hooks-dialog.glade",  
 				 NULL, NULL);
    assert(xml);
-   hook_dialog->glade_xml = xml;
+   priv->glade_xml = xml;
    glade_xml_signal_connect_data(xml, "on_button_next_clicked", 
 				 G_CALLBACK(cb_button_next), 
 				 (gpointer)ta);
